/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import News.*;
import utils.*;

public class GroupViewer extends JFrame implements Runnable, ActionListener, ListSelectionListener
{
	private NewsGroup group;
	private JMenu I_Reply;
	private JMenuItem I_Refresh, I_StopeR;
	private JList lst;
	private ArticleComposer viewer=null;
	private Boolean running=Boolean.FALSE;
	private Thread GroupReader;
	private NewsGroupList GroupList;
	private ArticleComposer Composer;
	private NewsArticle CurrentArticle;
	private boolean connected=false;
	private DefaultListModel listModel;
	private boolean stopRefresh;
	
	public GroupViewer(NewsGroupList groupList, NewsGroup g)
	{
		super(g.name());
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e)
            {
                Close();
            }
        });
		JMenu M1;
		JMenuBar Bar;

		GroupList=groupList;
		group=g;
		JPanel mainP=new JPanel();
		
		listModel=new DefaultListModel();
		lst=new JList(listModel);
		JScrollPane scroller=new JScrollPane(lst);
		lst.addListSelectionListener(this);
		mainP.add("Center", scroller);

		Bar=new JMenuBar();
		M1=new JMenu("Group");
		I_Refresh=new JMenuItem("Refresh");
		I_Refresh.addActionListener(this);
		M1.add(I_Refresh);
		I_StopeR=new JMenuItem("Stop Refresh");
		I_StopeR.setEnabled(false);
		I_StopeR.addActionListener(this);
		M1.add(I_StopeR);
		M1.addSeparator();

		JMenuItem jmi;
		M1.add(jmi=new JMenuItem("Subscribe"));
		jmi.addActionListener(this);

		M1.add(jmi=new JMenuItem("Unsubscribe"));
		jmi.addActionListener(this);

		M1.addSeparator();
		
		M1.add(jmi=new JMenuItem("Close"));
		jmi.addActionListener(this);

		Bar.add(M1);

		M1=new JMenu("Articles");

		I_Reply=new JMenu("Reply To");

		I_Reply.add(jmi=new JMenuItem("News"));
		jmi.addActionListener(this);
		I_Reply.add(jmi=new JMenuItem("Mail"));
		jmi.addActionListener(this);
		I_Reply.add(jmi=new JMenuItem("Mail & News"));
		jmi.addActionListener(this);

		M1.add(I_Reply);
		I_Reply.setEnabled(false);
		M1.add(jmi=new JMenuItem("New article"));
		jmi.addActionListener(this);
		Bar.add(M1);
		setJMenuBar(Bar);
		getContentPane().add("Center", mainP);
		
		pack();
		show();
	}

	protected void Close()
	{
	    if (viewer != null)
		    viewer.dispose();
		group.Inactive();
		dispose();
	}

	public void Refresh()
	{

		if ((GroupReader!=null)&&(GroupReader.isAlive()))
		{
			System.out.println("GroupReader already running");
			return;
		}
		GroupReader=new Thread(this, "Group Loader-"+new Date());
		GroupReader.setPriority(Thread.currentThread().getPriority()-1);
		System.out.println("Reader Thread :"+GroupReader);
		GroupReader.start();
	}

	public void actionPerformed(ActionEvent e)
	{
	    Object source=e.getSource();

		if (source instanceof JMenuItem)
		{
		    String cmd=e.getActionCommand();
		    try
		    {
			    if (cmd.equals("Refresh"))
			    {
				    Refresh();
			    }
			    else if (cmd.equals("Stop Refresh"))
			    {
					stopRefresh=true;
					if ((GroupReader!=null)&&GroupReader.isAlive())
						GroupReader.interrupt();
			    }
			    else if (cmd.equals("Subscribe"))
			    {
				    GroupList.subscribeTo(group);
			    }
			    else if (cmd.equals("Unsubscribe"))
			    {
				    GroupList.unsubscribeTo(group);
			    }
			    else if (cmd.equals("Close"))
			    {
				    Close();
			    }
			    else if (cmd.equals("Show all articles"))
			    {

				}
				else if (cmd.equals("News"))
				{
					Composer=new ArticleComposer(GroupList, group, CurrentArticle, ArticleComposer.NEWS|ArticleComposer.REPLY);
				}
				else if (cmd.equals("Mail"))
				{
					Composer=new ArticleComposer(GroupList, group, CurrentArticle, ArticleComposer.MAIL|ArticleComposer.REPLY);
				}
				else if (cmd.equals("Mail & News"))
				{
					Composer=new ArticleComposer(GroupList, group, CurrentArticle, ArticleComposer.NEWS|ArticleComposer.MAIL|ArticleComposer.REPLY);
				}
				else if (cmd.equals("New article"))
				{
					Composer=new ArticleComposer(GroupList, group, null, ArticleComposer.NEWS);
				}
			}
			catch (Exception ex)
			{
				System.out.println("Exception "+ex.getMessage());
			}
		}
	}

	public void valueChanged(ListSelectionEvent e)
	{
		int index=((JList)e.getSource()).getSelectedIndex();

		if (index != -1)
		{
			CurrentArticle=(NewsArticle)lst.getSelectedValue();
			group.setArticleRead(CurrentArticle);
			if (CurrentArticle != null)
			{
				I_Reply.setEnabled(true);
				if (viewer==null)
				{
					viewer=new ArticleComposer(GroupList, group, CurrentArticle, ArticleComposer.VIEW);
				}
				else
				{
					viewer.setArticle(CurrentArticle);
					if (viewer.isShowing()==false)
						viewer.show();
				}
			}
		}
		else
		{
			CurrentArticle=null;
			I_Reply.setEnabled(false);
			if (viewer!=null)
				viewer.clearText();
		}
	}

	public synchronized void run()
	{
		Cursor oldcur;

		if (!connected)
		{
			if (group.Active()==false)
			{
				new JNewsDialog(this, "Error", "Newsgroup "+group.name()+" is already active !");
				dispose();
			}
			else
				connected=true;
		}
		/*
		oldcur=getCursor();
		setCursor(Cursor.getPredefinedCursor(WAIT_CURSOR));
		I_Refresh.setEnabled(false);
		I_StopeR.setEnabled(true);*/
		try
		{
			group.Update();
			if (Thread.currentThread().isInterrupted()==false)
			{
				stopRefresh=false;
				SwingUtilities.invokeLater(new Runnable() 
									   {
										   public void run()
											   {
												   refreshList();
											   }
									   }
						   );
			}
		}
		catch(InterruptedException intex)
		{
		    new JNewsDialog(this, "Note", "Operation aborted");
		}
		catch(java.io.IOException io)
		{
			new JNewsDialog(this, "Error", "An I/O Error occured : "+io.getMessage());
		}
		catch(NntpException nntp)
		{
		    new JNewsDialog(this, "Error", "A NNTP Error occured : "+nntp.getMessage());
		}
		catch(Exception ex)
		{
			System.out.println("Exception ( list "+group.name()+" ) "+ex.getMessage()+" Thread :"+Thread.currentThread());
		}
		finally
		{
			GroupReader=null;
			//setCursor(oldcur);
		}
	}

	protected void refreshList()
	{
		System.out.println("Refresh list started "+Thread.currentThread());
		if (viewer!=null)
			viewer.clearText();
		listModel.clear();
		Enumeration enum=group.listArticles();
		while (enum.hasMoreElements()&&(!stopRefresh))
			listModel.addElement(enum.nextElement());
		I_Refresh.setEnabled(true);
		I_StopeR.setEnabled(false);
	}
	
}


