/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import News.*;

public class JavaNews extends JFrame implements Runnable, ListSelectionListener, TreeSelectionListener
{
	JList lst;
	JPanel cards;
	CardLayout cl;
	NewsGroupList groups;
	NewsGroup selected;
	GroupViewer groupviewer=null;
	JNewsProps Props=null;
	private HostsProperties HProperties;
	private IdentProperties IProperties;
	int Port;
	String Host, Home, FileSep;
	boolean ShowOnlySubscribed=true;
	private JMenuItem subscribe, unsubscribe, view, abort, checkGrps, checkNewGrps;
	public static final int FILL_ONLY=0, ALL_GROUPS=1, NEW_GROUPS=2;
	private int LoadType;
	private Thread Loader;
	NewsGroupTree grouptree;
	DefaultListModel listModel;

	/*
	 * Constructor, set-up the application's window
	 * 
	 */
	public JavaNews()
	{
		super("JavaNews ( Subscribed )");

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e)
            {
                dispose();
            }

            public void windowClosed(WindowEvent e)
            {
                Quit();
			}
        });

		FileSep=System.getProperty("file.separator");
		Home=System.getProperty("user.home")+FileSep+".JavaNews";
        File mydir=new File(Home);
        if (!mydir.exists())
            mydir.mkdirs();
		Props=new JNewsProps(Home+FileSep+".JNewsrc");

        cards=new JPanel();
        cards.setLayout((cl=new CardLayout()));
		JPanel P=new JPanel(new BorderLayout());
		listModel=new DefaultListModel();
		lst=new JList(listModel);
		JScrollPane scroller=new JScrollPane(lst);
		P.add("Center", scroller);
        lst.addListSelectionListener(this);

		//Set up the menus
		JMenuBar Bar=new JMenuBar();
		createMenus(Bar);
		setJMenuBar(Bar);

		cards.add(P, "lists");
		cards.add((grouptree=new NewsGroupTree()), "Groups");
		grouptree.addTreeSelectionListener(this);

		getContentPane().add("Center", cards);

		unsubscribe.setEnabled(false);
		subscribe.setEnabled(false);
		view.setEnabled(false);

		pack();
		show();
		
		Thread t=new Thread() 
		 {
			 public void run()
				 {
					 Start();
				 }
		 };
		t.setPriority(Thread.NORM_PRIORITY-1);
		t.start();
	}

	/**
	 * Application's entry point
	 *
	 */
	public static void main(String argv[])
	{
		new JavaNews();
	}

	protected void fillList()
	{
		SwingUtilities.invokeLater(new Runnable() 
							   {
								   public void run() 
									   {
										   doFillList();
									   }
							   }
							   );
	}

	protected void doFillList()
	{
		Enumeration enum;
			
		if (ShowOnlySubscribed)
		{
		    cl.show(cards, "lists");
			if (LoadType == NEW_GROUPS)
			{
				enum=groups.getNewGroupsList();
			}
			else
			{
				listModel.clear();
				enum=groups.getSuscribedGroupsList();
			}
			while(enum.hasMoreElements())
			    listModel.addElement(((NewsGroup)enum.nextElement()).name());
		}
		else
		{
			System.out.println("calling show tree");
            cl.show(cards, "Groups");
		}

	}

	/**
	 * handles a few time consuming operations
	 * like retrieving newsgroups from server
	 * building the tree & list ...
	 */
	public void run()
	{
		Cursor oldcur;
		int count=0;
		JNewsDialog dlg;
		boolean interrupted=false;

		oldcur=cards.getCursor();
		cards.setCursor(Cursor.getPredefinedCursor(WAIT_CURSOR));
		abort.setEnabled(true);
		checkGrps.setEnabled(false);
		checkNewGrps.setEnabled(false);
		try
		{
			switch(LoadType)
			{
				case ALL_GROUPS :
					count=groups.getAllGroups();
					new JNewsDialog(this, "News groups", count+" News group retrieved");
					grouptree.setGroups(groups, Host);
			        grouptree.forceBuild();
					break;

				case NEW_GROUPS :
					//TODO : update the tree !
					count=groups.getNewGroups();
					new JNewsDialog(this, "New groups", "There are "+count+" new groups");
					break;
			}
		}
		catch (InterruptedException err)
		{
		    interrupted=true;
		    switch(LoadType)
			{
				case ALL_GROUPS :
					new JNewsDialog(this, "News groups", "Operation aborted");
					break;

				case NEW_GROUPS :
					 new JNewsDialog(this, "New groups", "Operation aborted");
					 break;
			}
		}
		catch (NntpException nntp)
		{
		    new JNewsDialog(this, "A news error occured", "The news server responded :",nntp.getMessage());
		}
		catch (Exception ex)
		{
			System.out.println("JavaNews run : Exception "+ex.getMessage()+"\n");
			ex.printStackTrace();
		}
		finally
		{
		    if ((!interrupted)&&((count>0)||(LoadType==FILL_ONLY)))
		        fillList();
		    cards.setCursor(oldcur);
		    abort.setEnabled(false);
		    checkGrps.setEnabled(true);
		    checkNewGrps.setEnabled(true);
		    Loader=null;
		}
	 }

	protected void SaveState()
	{

		try
		{
			if (groups != null)
			{
	    		groups.toFile();
				groups.toNewsrcFile();
				//Free some resources to avoid a prematured exit
				groups=null;
				System.gc();
				grouptree.save();
			}
			Props.SaveProps();
		}
		catch (Exception ex2)
		{
			System.out.println("SaveState "+ex2.getMessage());
		}
	}

	protected void Start()
	{

		Port=Integer.valueOf(Props.getProperty("NntpPort")).intValue();
		Host=Props.getProperty("NntpHost");
		HProperties=new HostsProperties(Props, this);
		IProperties=new IdentProperties(Props, this);
		try
		{
			groups=new NewsGroupList(Host, Port, Home+FileSep+"JNewsgroups-"+Host+"_"+Port, Home+FileSep+"newsrc-"+Host+"_"+Port, Props);
			System.out.println("Done, starting tree");
			LoadType=FILL_ONLY;
			grouptree.setDumpFile(Home+FileSep+"JNewsgroups-"+Host+"_"+Port+".tree");
			grouptree.setGroups(groups, Host);
			grouptree.restore();
			fillList();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			new JNewsDialog(this, "Error", "An error occured", ex.getMessage());
		}
	}

	/**
	 * Terminates the application after saving the current state
	 *
	 */
	void Quit()
	{
		try
		{
			SaveState();
			dispose();
		}
		catch (Exception ex2)
		{
			System.out.println("Terminate "+ex2.getMessage());
		}
		finally
		{
			System.exit(0);
		}
	}

	/**
	 * When the properties have changed ( nntp host , smtp hosts ... )
	 * Close the current session ( save the current state ) and restart
	 * a new session with the new parameters
	 */
	public void PropsChanged()
	{

		try
		{
			SaveState();
		}
		catch (Exception ex2)
		{
			System.out.println("PropsChanged Save "+ex2.getMessage());
		}
		(new Thread() 
								   { 
									   public void run ()
									   { Start(); }
								   }).start();
	}

	/**
	 * Set up the menus ...
	 */
	protected void createMenus(JMenuBar Bar)
	{
		JMenuItem jmi;
		JMenu M1, M2;

		M1=new JMenu("News Groups");
		jmi=new JMenuItem("Show Subscribed groups");
		jmi.addActionListener(new ActionListener() 
							{
								public void actionPerformed(ActionEvent e)
									{
										showSubscribedGroups();
									}
							}
							);
		
								
		M1.add(jmi);
		jmi=new JMenuItem("Show All groups");
		jmi.addActionListener(new ActionListener() 
							{
								public void actionPerformed(ActionEvent e)
									{
										showAllGroups();
									}
							}
							);
		
								
		M1.add(jmi);
		M1.addSeparator();
		checkNewGrps=new JMenuItem("Check for new Groups");
		checkNewGrps.addActionListener(new ActionListener() 
							{
								public void actionPerformed(ActionEvent e)
									{
										checkNewGroups();
									}
							}
							);
		M1.add(checkNewGrps);
		checkGrps=new JMenuItem("Retrieve all Groups");
		checkGrps.addActionListener(new ActionListener() 
							{
								public void actionPerformed(ActionEvent e)
									{
										retrieveAllGroups();
									}
							}
							);
		M1.add(checkGrps);
		abort=new JMenuItem("Abort retrieval operation");
		abort.addActionListener(new ActionListener() 
							{
								public void actionPerformed(ActionEvent e)
									{
										abort();
									}
							}
							);
		M1.add(abort);
		abort.setEnabled(false);

		M1.addSeparator();
		jmi=new JMenuItem("Quit");
		jmi.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e)
									{
										Quit();
									}
							}
							);
		M1.add(jmi);
		Bar.add(M1);

		M1=new JMenu("Group");
		view=new JMenuItem("View");
		view.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e)
									{
										viewGroup();
									}
							}
							);
		M1.add(view);
		M1.addSeparator();
		subscribe=new JMenuItem("Subscribe");
		subscribe.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e)
									{
										subscribe();
									}
							}
							);
		
								
		M1.add(subscribe);
		
		unsubscribe=new JMenuItem("Unsubscribe");
		unsubscribe.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e)
									{
										unSubscribe();
									}
							}
							);		
		M1.add(unsubscribe);
		Bar.add(M1);

		M2=new JMenu("Options");
		M1=new JMenu("Configuration");

		jmi=new JMenuItem("Hosts");
		jmi.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e)
									{
										configureHosts();
									}
							}
							);
		M1.add(jmi);

		jmi=new JMenuItem("Identity");
		jmi.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e)
									{
										configureIdentity();
									}
							}
							);
		M1.add(jmi);
		M2.add(M1);
		Bar.add(M2);

		JMenu Help=new JMenu("Help");
		jmi=new JMenuItem("About");
		jmi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
									{
										about();
									}
							}
							);
		
		Help.add(jmi);
		//Bar.setHelpMenu(Help); //not implemented !
		Bar.add(Help);
	}
	

	
   
	protected void abort()
	{
		if ((Loader!=null)&&(Loader.isAlive()))
			Loader.interrupt();
	}

	protected void viewGroup()
	{
		groupviewer=new GroupViewer(groups, selected);
		//groupviewer.Refresh();
	}

	protected void retrieveAllGroups()
	{
		LoadType=ALL_GROUPS;
		ShowOnlySubscribed=false;
		Loader=new Thread(this, "Groups loader");
		//Loader.setPriority(Thread.MAX_PRIORITY-1);
		Loader.start();
	}
	
	protected void checkNewGroups()
	{
		LoadType=NEW_GROUPS;
		Loader=new Thread(this, "NewGroups loader");
		//Loader.setPriority(Thread.MAX_PRIORITY-1);
		Loader.start();
	}
	
	protected void subscribe()
	{
		groups.subscribeTo(selected);
	}

	
	protected void unSubscribe()
	{
		groups.unsubscribeTo(selected);
	}
	

	protected void showAllGroups()
	{
		ShowOnlySubscribed=false;
		super.setTitle("Java News ( All groups )");
		LoadType=FILL_ONLY;
		Loader=new Thread(this, "ListFiller");
		//Loader.setPriority(Thread.MAX_PRIORITY-1);
		Loader.start();
	}

	protected void showSubscribedGroups()
	{
		ShowOnlySubscribed=true;
		LoadType=FILL_ONLY;
		super.setTitle("Java News ( Subscribed )");
		Loader=new Thread(this, "ListFiller");
		//Loader.setPriority(Thread.MAX_PRIORITY-1);
		Loader.start();
	}

	protected void configureHosts()
	{
		HProperties.show();
	}

	protected void configureIdentity()
	{
		IProperties.show();
	}
	protected void about()
	{
		new AboutDialog(this, Props.getProperty("Version"));
	}

	/**
	 * called when an item is selected/deselected from the group list
	 *
	 * TODO : handle double click to select 
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		boolean failed=false;
		String name;

		int index=((JList)e.getSource()).getSelectedIndex();
		if (index != -1)
		{
			name=(String)lst.getSelectedValue();
			try
			{
				selected=groups.getByName(name);
			}
			catch (Exception ex)
			{
				System.out.println("JNews select Exception : "+ex.getMessage());
				failed=true;
			}
			if (!failed)
			{
				unsubscribe.setEnabled(true);
				subscribe.setEnabled(true);
				view.setEnabled(true);
			}
		}
		else
		{
			selected=null;
			unsubscribe.setEnabled(false);
			subscribe.setEnabled(false);
			view.setEnabled(false);
		}

	}

	/**
	 * Called when a selection/deselection
	 * happens in the Tree representing the
	 * full newsgroups hierachy
	 */
	public void valueChanged(TreeSelectionEvent e)
	{

        Object node=e.getPath().getLastPathComponent();
        if (node instanceof DefaultMutableTreeNode)
        {
            DefaultMutableTreeNode dnode=(DefaultMutableTreeNode)node;
			//If it's not a leaf, do nothing
            if (dnode.getAllowsChildren())
                return;
            String name=(String)dnode.getUserObject();
            if (e.isAddedPath(e.getPath()))
            {
                boolean failed=false;
                try
			    {
				    selected=groups.getByName(name);
			    }
			    catch (Exception ex)
			    {
				    System.out.println("JNews select Exception : "+ex.getMessage());
				    failed=true;
			    }
			    if (!failed)
			    {
				    unsubscribe.setEnabled(true);
				    subscribe.setEnabled(true);
				    view.setEnabled(true);
			    }
               System.out.println("Selected "+name+" length="+name.length());
            }
            else
            {
			    selected=null;
			    unsubscribe.setEnabled(false);
			    subscribe.setEnabled(false);
			    view.setEnabled(false);
			    System.out.println("Deselected "+name);
		    }
        }
	}
}
