/************************************************************************************
* Copyright (C) 1998 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package Message;
import java.lang.*;
import java.util.*;

public class Body
{
  protected static boolean debug=false;
  protected String _body;
  protected Vector _parts;
  protected String _type;
  protected String _subtype;

  public Body(Header header, String body) throws MimeException
  {
    _body=new String(body);
    String ctype=header.getField(Header.Content_Type);
    if (ctype!=null)
    {
      //throw new MimeException("Missing Content-type header in message");
      Hashtable value=Header.parseValue(ctype);
      _type=(String)value.get(Header.Type);
      _subtype=(String)value.get(Header.Subtype);
      if (Header.Multipart.equals(_type))
	extractParts("--"+Header.unQuote((String)value.get(Header.Boundary)));
      else
	throw new MimeException("unable to handle mime-type "+_type+"/"+_subtype);
    }
    else if (debug)
      System.out.println("No content-type in message ...");
  }

  public Body(String body)
  {
    _body=new String(body);
  }

  public Body()
  {
  }

  protected void extractParts(String boundary) throws MimeException
  {
    if (boundary==null)
      throw new MimeException("boundary is null. Unable to extract parts from message");
    if (debug)
      System.out.println("parsing body with boundary= "+boundary+"\n");
    int idx_b=_body.indexOf(boundary);//skip preamble
    int end=_body.indexOf(boundary+"--"); // find end marker
    int idx_e=0;
    int step=boundary.length()+1;//take the ending \n into account
    MimePart cur;
    if (idx_b==-1)
      throw new MimeException("boundary "+boundary+" not found in body");
    if (end==-1)
      throw new MimeException("ending boundary "+boundary+"-- not found in body");
    _parts=new Vector();
    idx_b+=step;
    while(idx_b<end)
    {
      idx_e=_body.indexOf(boundary, idx_b);
      if (idx_e==-1)
	throw new MimeException("Unterminated part in body ( missing boundary @ end )");
      else
      {
	cur=new MimePart(_body.substring(idx_b, idx_e-1));
	idx_b=idx_e+step;
      }
      _parts.addElement(cur);
    }
    _body=null; //don't keep the original body.
    if (debug)
      System.out.println("found "+_parts.size()+" parts in body");
  }

  public void addPart(MimePart part)
  {
    if (_parts==null)
      _parts=new Vector();
    _parts.addElement(part);
  }
  
  public void removePart(MimePart part)
  {
    if (_parts!=null)
      _parts.removeElement(part);
  }
  
  public Enumeration getParts()
  {
    if (_parts!=null)
      return _parts.elements();
    else
      return null;
  }

  public boolean hasParts()
  {
    if ((_parts!=null)&&_parts.size()>0)
      return true;
    else
      return false;
  }
  
  public StringBuffer buildUp() throws MimeException
  {
    if (hasParts())
      throw new MimeException("Unable to generate Mime body, no boundary defined");
    //simple message, just return the body
    return new StringBuffer(_body);
  }

  public StringBuffer buildUp(String boundary)
  {
    if (!hasParts())
    {
      //simple message, just return the body
      return new StringBuffer(_body);
    }
    else
    {
      //build up the body, using the specified boundary.
      boundary="--"+boundary;
      StringBuffer sb=new StringBuffer("This is a Mime message\n");
      Enumeration parts=getParts();
      while(parts.hasMoreElements())
      {
	sb.append('\n');
	sb.append(boundary);
	sb.append('\n');
	sb.append(((MimePart)parts.nextElement()).toString());
      }
      sb.append('\n');
      sb.append(boundary);
      sb.append("--\n");
      return sb;
    }
  }

}







