/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package News;

class Interval extends Object {
	private int inf;
	private int sup;


	public Interval(int val) {
		inf=val;
		sup=val;
	}

	public Interval(int val1, int val2) {
		if (val1 <= val2) {
			inf=val1;
			sup=val2;
		}
		else {
			sup=val1;
			inf=val2;
		}
	}

	public boolean contiguouslyGreaterThan(Interval interval) {
		return(inf()==interval.sup()+1 || inf()==interval.sup());
	}

	public boolean contiguouslyLesserThan(Interval interval) {
		return(sup()+1==interval.inf() || sup()==interval.inf());
	}

	public boolean lesserThan(Interval interval) {
		return(sup() <= interval.inf());
	}

	public boolean greaterThan(Interval interval) {
		return(interval.sup() <= inf());
	}

	public boolean contiguous(Interval interval) {
		return(		contiguouslyGreaterThan(interval) ||
				 	contiguouslyLesserThan(interval));
	}

	public boolean within(Interval interval) {
		return(	interval.inf() <= inf() && interval.sup() >= sup());
	}

	public Interval(Interval interval1, Interval interval2) throws Exception {
		if (interval1.contiguouslyLesserThan(interval2)) {
			inf=interval1.inf();
			sup=interval2.sup();
		}
		else if (interval1.contiguouslyGreaterThan(interval2)) {
			inf=interval2.inf();
			sup=interval1.sup();
		}
		else throw new Exception("Tried to merge non-contiguous intervals");
		//System.err.println("Merged interval "+this);
	}

	public int sup() {
		return sup;
	}

	public int inf() {
		return inf;
	}

	public String toString() {
		if (inf==sup) return(""+inf);
		else return(inf+"-"+sup);
	}
}
