/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package News;
import java.lang.*;
import java.util.*;
import java.io.*;
import News.NewsGroup;
import News.NewsServer;
import News.JNewsProps;
import utils.*;

public class NewsGroupList
{
	private String Host;
	private int Port;
	private Hashtable AllGroups, NewGroups;
  	private OrderedFolder OrderedList, SubscribedOrderedList;
	private NewsGroup selectedGroup;
	private String ResourceFile, newsrcFile;
	private boolean Updated=false;
	private JNewsProps Properties;
	private boolean firstsave=false;

	public NewsGroupList(String host, int port, String resourceFile, String newsrc, JNewsProps props)
	{

		Host=host;
		Port=port;
		Properties=props;
		AllGroups=new Hashtable();
		SubscribedOrderedList=new OrderedFolder();
		OrderedList=new OrderedFolder();
		selectedGroup=null;
		ResourceFile=resourceFile;
		newsrcFile=newsrc;
		getFromFile();
		getFromNewsrcFile();
	}

	private String TwoDigits(int val)
	{
		String ret;

		if (val<10)
			ret=new String("0"+Integer.toString(val));
		else
			ret=Integer.toString(val);
		return ret;
	}

	private String SetLastUpdate()
	{
		Date now;
		String date;

		now=new Date();
		date=new String(TwoDigits(now.getYear())+TwoDigits(now.getMonth()+1)+TwoDigits(now.getDate())+" "+TwoDigits(now.getHours())+TwoDigits(now.getMinutes())+TwoDigits(now.getSeconds()));
		Properties.put("LastUpdate", date);
		return date;
	}

	public JNewsProps getProperties()
	{

		return Properties;
	}

	public void addGroup(NewsGroup group)
	{

		AllGroups.put(group.name(), group);
		OrderedList.insert(group);
		if (group.subscribed())
		{
			SubscribedOrderedList.insert(group);
		}
	}

	public int getAllGroups() throws Exception
	{
		String liste, Nom;
		StringTokenizer group, params;
		NewsGroup next, cur;
		int first, last, count=0;
		boolean interrupted=false;
		NewsServer Serveur=new NewsServer(Host, Port);

		Serveur.connect();
		try
		{
			liste=Serveur.List();
			group=new StringTokenizer(liste, "\n");
			while (group.hasMoreTokens())
			{
				liste=group.nextToken();
				params=new StringTokenizer(liste, " ");
				Nom=params.nextToken();
				last=(new Integer(params.nextToken())).intValue();
				first=(new Integer(params.nextToken())).intValue();
				if ((cur=(NewsGroup)AllGroups.get(Nom))!=null)
				{
					cur.setIndexes(first, last);
				}
				else
				{
					next=new NewsGroup(Host, Port, Nom, last, first, params.nextToken());
					AllGroups.put(next.name(), next);
					OrderedList.insert(next);
				}
				count++;
				//Check is this necessary ?
				//java.lang.Thread.yield(); 
			}
		}
		catch (InterruptedException interr)
		{
		    interrupted=true;
		    throw (interr);
		}
		finally
		{
			if (!interrupted)
			{
			    Updated=true;
			    SetLastUpdate();
			}
			Serveur.disconnect();
		}
		return count;
	}


	public int getNewGroups() throws Exception
	{
		String liste, Nom, lastupdate;
		StringTokenizer group, params;
		NewsGroup next, cur;
		int first, last, count=0;
		boolean interrupted=false;
		NewsServer Serveur=new NewsServer(Host, Port);

		Serveur.connect();
		try
		{
			lastupdate=(String)Properties.getProperty("LastUpdate", "none");
			if (lastupdate.equals("none"))
				return 0;
			liste=Serveur.NewGroups(lastupdate);
			group=new StringTokenizer(liste, "\n");
			NewGroups=new Hashtable();
			while (group.hasMoreTokens())
			{
				count++;
				liste=group.nextToken();
				params=new StringTokenizer(liste, " ");
				Nom=params.nextToken();
				last=(new Integer(params.nextToken())).intValue();
				first=(new Integer(params.nextToken())).intValue();
				if ((cur=(NewsGroup)AllGroups.get(Nom))!=null)
				{
					//System.out.println("New group ( pre-existant ) : "+Nom);
					cur.setIndexes(first, last);
				}
				else
				{
	    			//System.out.println("New group : "+Nom);
		    		next=new NewsGroup(Host, Port, Nom, last, first, params.nextToken());
					AllGroups.put(next.name(), next);
					NewGroups.put(next.name(), next);
					OrderedList.insert(next);
				}
				//Check is this necessary ?
				//java.lang.Thread.yield();
			}
		}
		catch (InterruptedException interr)
		{
		    interrupted=true;
		    throw (interr);
		}
		finally
		{
	        if (!interrupted)
			{
			    Updated=true;
			    SetLastUpdate();
			}
			Serveur.disconnect();
		}
		return count;
	}

	public int getGroupCount()
	{

		return AllGroups.size();
	}

	public int getSuscribedGroupCount()
	{

		return SubscribedOrderedList.size();
	}

	public Enumeration getAllGroupsList()
	{

		return OrderedList.elements();
	}

	public Enumeration getSuscribedGroupsList()
	{

		return SubscribedOrderedList.elements();
	}

	public Enumeration getNewGroupsList()
	{

		return NewGroups.elements();
	}

	public NewsGroup getByName(String name)
	{

		return (NewsGroup)AllGroups.get(name);
	}

	protected void write(PrintWriter out)
	{
		Enumeration enum=OrderedList.elements();
		while (enum.hasMoreElements())
			out.println(((NewsGroup)enum.nextElement()).name());
	}

	public String toString()
	{
		Enumeration enum=OrderedList.elements();
		StringBuffer buff=new StringBuffer("");
		NewsGroup group;

		while (enum.hasMoreElements())
		{
			group=(NewsGroup)enum.nextElement();
			buff.append(group.name());
			buff.append('\n');
		}
		return buff.toString();
	}

	public void getFromFile()
	{
		File src;
		BufferedReader in=null;
		String line;
		
		System.out.println("Retrieving group list from file "+ResourceFile);
		src=new File(ResourceFile);
		try
		{
			if (!(src.exists()&&src.canRead()))
			{
				firstsave=true;
				return;
			}
			in=new BufferedReader(new FileReader(src));
			while((line=in.readLine())!=null)
				addGroup(new NewsGroup(Host, Port, line));
		}
		catch (IOException ex)
		{
			System.out.println("Exception while reading "+ResourceFile+" : "+ex.getMessage());
			return;
		}
		finally
		{
			if (in != null)
			{
				try { in.close(); }
				catch (IOException e1) {}
			}
		}
		System.out.println(getGroupCount()+" groups retrieved");
	}

	public void toFile() throws Exception
	{
		File dst;
		PrintWriter out;

		if (Updated==false)
			return;
		System.out.println("Saving group list to file "+ResourceFile);
		dst=new File(ResourceFile);
		if (dst.exists()&&(!dst.canWrite()))
		{
			System.out.println("permission to write to "+ResourceFile+" denied !");
			return;
		}
		out=new PrintWriter(new BufferedWriter(new FileWriter(dst)));
		write(out);
		out.close();
	}

	public void toNewsrcFile() throws Exception
	{
		File dst;
		BufferedWriter out;
		String value;
		int length;
		Enumeration enum=AllGroups.elements();
		StringBuffer buff=new StringBuffer("");
		NewsGroup group;

		while (enum.hasMoreElements())
		{
			group=(NewsGroup)enum.nextElement();
			if (group.hasRead()||group.subscribed())
				buff.append(group.toString()+"\n");
		}
		value=buff.toString();
		length=value.length();
		if ((length==0)&&(firstsave==true))
			return;
		System.out.println("Saving group list to file "+newsrcFile);
		dst=new File(newsrcFile);
		if (dst.exists()&&(!dst.canWrite()))
		{
			System.out.println("permission to write to "+newsrcFile+" denied !");
			return;
		}
		out=new BufferedWriter(new FileWriter(dst));
		out.write(value);
		out.close();
	}

	public void getFromNewsrcFile()
	{
		File src;
		BufferedReader in=null;
		String Desc, cur;
		boolean subscribed;
		NewsGroup group;
		int index;

		System.out.println("Retrieving newsrc from file "+newsrcFile);
		src=new File(newsrcFile);
		try
		{
			if (!(src.exists()&&src.canRead()))
				return;
			in=new BufferedReader(new FileReader(src));
			while((cur=in.readLine())!=null)
			{
				index=cur.indexOf(':');
				if (index == -1)
				{
					index=cur.indexOf('!');
					if (index == -1)
					{
						System.out.println("Corrupted newsrc entry : "+cur);
						continue;
					}
					subscribed=false;
				}
				else
					subscribed=true;
				group=(NewsGroup)AllGroups.get(cur.substring(0, index));
				if (group != null)
				{
					if (subscribed)
						subscribeTo(cur.substring(0, index));
					group.setReadList(cur.substring(index+1), subscribed);
				}
			}
		}
		catch (IOException ex)
		{
			System.out.println("Exception while reading "+newsrcFile+" : "+ex.getMessage());
		}
		finally
		{
			if (in != null)
				try 
				{ 
					in.close(); 
				}
			catch (IOException e1) {}
		}
	}

	public void subscribeTo(String name)
	{
		NewsGroup G;

		G=(NewsGroup)AllGroups.get(name);
		if (G.subscribed())
			return;
		G.subscribe();
		SubscribedOrderedList.insert(G);
	}

	public void subscribeTo(NewsGroup group)
	{

		if (group.subscribed())
			return;
		group.subscribe();
		SubscribedOrderedList.insert(group);
	}

	public void unsubscribeTo(String name)
	{
		NewsGroup G;

		G=(NewsGroup)AllGroups.get(name);
		if (!G.subscribed())
			return;
		G.unsubscribe();
		SubscribedOrderedList.removeElement(G);
	}

	public void unsubscribeTo(NewsGroup group)
	{

		if (!group.subscribed())
			return;
		group.unsubscribe();
		SubscribedOrderedList.removeElement(group);
	}
}
