/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package News;
import java.net.*;
import java.lang.*;
import java.io.*;
import java.util.*;
import News.NntpException;
import News.NntpResponse;

public class NewsServer
{
	private int Port=119;
	private String NewsHost;
	private InetAddress HostAddr;
	private Socket Sock;
	private BufferedReader In;
	private PrintWriter Out;
	protected boolean connected=false;

	public void connect() throws IOException, NntpException
	{

		HostAddr=InetAddress.getByName(NewsHost);
		Sock=new Socket(HostAddr, Port);
		In = new BufferedReader(new InputStreamReader(Sock.getInputStream()));
		Out=new PrintWriter(new BufferedOutputStream(Sock.getOutputStream()));
		if ((getResponse().getMessage()).indexOf("INN")!=-1)
		{
		  Out.print("mode reader\r\n");
		  Out.flush();
		  if (getResponse().getCode()!=200)
		    throw new NntpException("Problem with your news server ...");
		}
		System.out.println("Connected to : "+HostAddr+":"+Port);
		connected=true;
	}

	public void disconnect()
	{

        if (connected==false)
            return;
		System.out.println("Deconnection");
		try
		{
			if (In != null)
			{
				In.close();
				In=null;
			}
	    	if (Out != null)
			{
			    Out.close();
			    Out=null;
			}
		    if (Sock != null)
		    {
			    Sock.close();
			    Sock=null;
			}
		}
		catch(Exception ex)
		{
			System.out.println("Exception "+ex.getMessage());
		}
		finally
		{
		    connected=false;
		}
	}

	private String readText() throws Exception
	{
		String Line;
		StringBuffer buff=new StringBuffer();

		while ((!(Line = In.readLine()).equals("."))&&(!Thread.currentThread().isInterrupted()))
			buff=buff.append(Line+"\n");
		if (Thread.currentThread().isInterrupted())
		    throw new InterruptedException("Operation aborted");
		return buff.toString();
	}

	private String readBody() throws Exception
	{
		String Line;
		StringBuffer buff=new StringBuffer();

		while (!(Line = In.readLine()).equals("."))
		{
			if ((Line.length()>0)&&(Line.charAt(0)=='.'))
				buff.append(Line.substring(1)+"\n");
			else
				buff=buff.append(Line+"\n");
		}
		return buff.toString();
	}

	public NewsServer(String host, int port) throws IOException
	{

		NewsHost=host;
		Port=port;
	}

	public NewsServer(String host)
	{

		NewsHost=host;
	}

	private NntpResponse getResponse() throws NntpException, IOException
	{
		int status;
		NntpResponse resp;
		String line=In.readLine();
	       
		if (line == null)
			throw(new IOException("Connection closed"));
		//System.out.println("==>resp :"+line);
		resp=new NntpResponse(line);
		status=resp.getCode();
		if ((status >= 100 && status <= 299)||(status == 340))
			return resp;
		throw new NntpException(resp.getMessage());
	}

	public String selectGroup(String name) throws java.io.IOException, NntpException
	{
		int count;

		while (In.ready())
		{
			System.out.println("Skipping 1 byte");
			In.skip(1);
		}
		Out.print("group ");
		Out.print(name);
		Out.print("\r\n");
		Out.flush();
		return getResponse().getMessage();
	}

	public void selectArticle(int index) throws Exception
	{

		Out.print("stat ");
		Out.print(index);
		Out.print("\r\n");
		Out.flush();
		getResponse();
	}

	public String getHeader(int index) throws Exception
	{

		Out.print("head ");
		Out.print(index);
		Out.print("\r\n");
		Out.flush();
		getResponse();
		return readText();
	}

	public String getHeader(String Id) throws Exception
	{

		Out.print("head ");
		Out.print(Id);
		Out.print("\r\n");
		Out.flush();
		getResponse();
		return readText();
	}

	public String getHeader() throws Exception
	{
		String retour;

		Out.print("head\r\n");
		Out.flush();
		getResponse();
		return readText();
	}

	public String getBody(int index) throws Exception
	{
		String retour;

		Out.print("body ");
		Out.print(index);
		Out.print("\r\n");
		Out.flush();
		getResponse();
		return readBody();
	}

	public String getBody(String Id) throws Exception
	{

		Out.print("body ");
		Out.print(Id);
		Out.print("\r\n");
		Out.flush();
		getResponse();
		return readBody();
	}

	public String getBody() throws Exception
	{

		Out.print("body\r\n");
		Out.flush();
		getResponse();
		return readBody();
	}

	public String Next() throws Exception
	{

		Out.println("next\r\n");
		Out.flush();
		return getResponse().getMessage();
	}

	public void Last() throws Exception
	{

		Out.print("last\r\n");
		Out.flush();
		getResponse();
	}

	public void Quit() throws Exception
	{

		Out.print("quit\r\n");
		Out.flush();
		In.readLine();
	}

	public String List() throws Exception
	{
		Out.print("list\r\n");
		Out.flush();
		getResponse();
		return readText();
	}

	public String NewGroups(String Date) throws Exception
	{

		Out.println("newgroups "+Date);
		Out.flush();
		getResponse();
		return readText();
	}

	public String getID()
	{
		String Host;
		InetAddress addr;
		String ID, Dt;
		Random rand=new Random();
		Date date=new Date();

		try
		{
			addr=InetAddress.getLocalHost();
			Host=addr.getHostName();
		}
		catch (UnknownHostException ex)
		{
			System.out.println("Unable to get local address : "+ex.getMessage());
			Host="NoHostnameFound";
		}
		ID=new String("<JNews-"+date.getTime()+"-"+Math.abs(rand.nextInt()/100)+"@"+Host+">");
		//System.out.println("New ID = "+ID);
		return ID;
	}

	private String ProcessBody(String Body)
	{
		StringBuffer buff=new StringBuffer("");
		String tmp;
		StringTokenizer token;

		token=new StringTokenizer(Body, "\n", true);
		while (token.hasMoreElements())
		{
			tmp=token.nextToken();
			//System.out.println("Token : "+tmp+" Fin Token");
			if (tmp.startsWith("."))
				buff.append("."+tmp);
			else
				buff.append(tmp);
		}
		return buff.toString();
	}

	public boolean Post(String Header, String Body) throws NntpException, IOException
	{
		int code;
		String NewBody;

		NewBody=ProcessBody(Body);
		try
		{
		    Out.println("post");
		    Out.flush();
		    code=getResponse().getCode();
		    if (code==340)
			{
			    Out.print(Header);
			    Out.print("\r\n");
			    Out.print(NewBody);
			    Out.print("\r\n.\r\n");
			    Out.flush();
			    code=getResponse().getCode();
			    //System.out.println("Message posted, code = "+code);
			    return (code==240);
			}
		}
		catch (NntpException e)
		{
			System.out.println("Exception Post = "+e.getMessage());
			System.out.println("\nHeader : \n"+Header);
			System.out.println("\nBody : \n"+NewBody);
			throw e;
		}
		return false;
	}

	protected void finalize() throws Throwable
	{

		disconnect();
	}
}


