/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import java.awt.*;
import java.io.*;
import News.*;


public class NewsGroupTree extends JPanel implements Runnable
{
    NewsGroupList groups;
    String name;
    DefaultMutableTreeNode root;
    JTree tree;
    JScrollPane scrollpane;
    Vector listeners;
	CardLayout cards;
	JProgressBar progress;
	String dumpFile;
	int buildMode;
	protected boolean modified=false;
	protected static final int Restore=1;
	protected static final int Rebuild=2;
	
    public NewsGroupTree()
    {
        super();
        listeners=new Vector();
		setLayout((cards=new CardLayout()));
		progress=new JProgressBar(0, 100);
		add(progress, "Progress");
		scrollpane=new JScrollPane();
		add(scrollpane, "Tree");
		cards.show(this, "Progress");
    }

    public NewsGroupTree(NewsGroupList list, String name)
    {
        this();
        this.groups=list;
        this.name=name;
    }

    public void setGroups(NewsGroupList list, String name)
    {
        this.groups=list;
        this.name=name;
    }

    public synchronized void forceBuild()
    {
		buildMode=Rebuild;
        Thread processor=new Thread(this, "Tree-builder");
        processor.setPriority(Thread.NORM_PRIORITY-1);
        processor.start();
    }

	public void setDumpFile(String s)
	{
		dumpFile=s;
	}

	public synchronized void restore()
	{
		buildMode=Restore;
		Thread processor=new Thread(this, "Tree-builder");
        processor.setPriority(Thread.NORM_PRIORITY-1);
        processor.start();
	}
	
	/**
	 * Serializes the tree in order to speed up startup
	 */
	public synchronized void save()  throws Exception
	{
		if (tree==null)
			return;
		if (dumpFile==null)
			throw new Exception("Cannot save tree , no dumpfile specified");
		//nothing to save if tree was not changed during session
		if (modified==false)
			return;
		ObjectOutputStream out=null;
		FileOutputStream fout=null;
		try
		{
			System.out.println("Start saving tree : "+(new Date()));
			out=new ObjectOutputStream((fout=new FileOutputStream(dumpFile)));
			out.writeObject(root);
			System.out.println("Tree saved !"+(new Date()));
		}
		catch (Exception e)
		{
			System.out.println("Error while saving tree : "+e);
			try
			{
				File f=new File(dumpFile);
				if (f.exists())
					f.delete();
			}
			catch (Exception e1)
			{
			}
		}
		finally 
		{
			if (fout!=null)
			{
				try
				{
					fout.close();
				}
				catch(Exception e1)
				{
				}
			}
		}
		
	}

    protected String[] getPath(String name)
    {
        StringTokenizer st=new StringTokenizer(name, ".");
        int count=st.countTokens();
        String[] res=new String[count];
        for (int i=0; i<count; i++)
            res[i]=st.nextToken();
        return res;
    }

	/**
	 * Insert an item into the tree
	 * This needs to be optimized/changed
	 * Far too slow whith a huge newsgroup list
	 */
    protected synchronized DefaultMutableTreeNode insertIntoTree(String[] path, String name)
    {
        int idx=0;
        boolean found=false;
        StringBuffer cPath=new StringBuffer(name.length());
        String curPath="";
        DefaultMutableTreeNode cur=root, tmp;
        Enumeration children;

        while(idx<path.length)
        {
            if (idx!=0)
                cPath.append(".");
            cPath.append(path[idx]);
            curPath=cPath.toString()+".*";
            children=cur.children();
            found=false;
            while(children.hasMoreElements())
            {
                tmp=(DefaultMutableTreeNode)children.nextElement();
                if (((String)tmp.getUserObject()).equals(curPath))
                {
                    found=true;
                    cur=tmp;
                    break;
                }
            }
            if (!found)
            {
                if (idx!=path.length-1)
                    tmp=new DefaultMutableTreeNode(curPath);
                else
                {
                    tmp=new DefaultMutableTreeNode(name);
                    tmp.setAllowsChildren(false);
                }
                cur.add(tmp);
                cur=tmp;
            }
            idx++;
        }
        return cur;
    }

    public void addTreeSelectionListener(TreeSelectionListener tsl)
    {
        listeners.addElement(tsl);
         if (tree!=null)
            tree.addTreeSelectionListener(tsl);
    }

    public void removeTreeSelectionListener(TreeSelectionListener tsl)
    {
        listeners.removeElement(tsl);
        if (tree!=null)
            tree.removeTreeSelectionListener(tsl);
    }

	protected boolean treeWasSaved()
	{
		File f=new File(dumpFile);
		return (f.exists()&&f.canRead()&&f.canWrite());
	}
	
	/*
	 * If the tree was previously serialized
	 * restore it by de-serializing it.
	 */
	protected boolean restoreTree()
	{
		ObjectInputStream in=null;
		BufferedInputStream fin=null;
		
		try
		{
			
			in=new ObjectInputStream((fin=new BufferedInputStream(new FileInputStream(dumpFile))));
			System.out.println("Restoring tree :"+(new Date()));
			root=(DefaultMutableTreeNode)in.readObject();
			tree=new JTree(root);
			System.out.println("Tree restored ..."+(new Date()));
			return true;
		}
		catch (Exception e)
		{
			System.out.println("Error restoring tree : "+e);
			return false;
		}
		finally
		{
			if (fin!=null)
			{
				try
				{
					fin.close();
				}
				catch(Exception e1)
				{
				}
			}
		}
	}

	/**
	 * rebuilds the tree from the underlying newsgroup list.
	 * This takes a lot of time.
	 * TODO : optimize/change the algorithm
	 */ 
	protected void buildFromScratch()
	{
		int idx=0;
		int tmpidx=0;
		Enumeration lst=groups.getAllGroupsList();
        root=new DefaultMutableTreeNode(name);
        String cur;
        DefaultMutableTreeNode node;
		
		System.out.println("Bulding tree from scratch "+(new Date()));
		modified=true;//ensure that the tree will be serialized when the session ends.
		progress.setMaximum(groups.getGroupCount());
		progress.setValue(0);
		cards.show(this, "Progress");
        while(lst.hasMoreElements())
		{
			if (++tmpidx==10)
			{
				tmpidx=0;
				idx+=10;
				progress.setValue(idx);
			}
			cur=((NewsGroup)lst.nextElement()).name();
			insertIntoTree(getPath(cur), cur);
		}
        tree=new JTree(root);
		System.out.println("Done !"+(new Date()));
	}
	
	/**
	 * figure out what to do to build the tree
	 * either de-serialize if the tree was previously serialized
	 * Or build a new tree if required
	 * All this is done in a separate Thread.
	 */
    public synchronized void run()
    {

		Cursor oldCur=getCursor();
		setCursor(Cursor.getPredefinedCursor(Frame.WAIT_CURSOR));
		if ((buildMode==Restore)&&treeWasSaved())
		{
			if (restoreTree()==false)
				buildFromScratch();
		}
		else
			buildFromScratch();
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		Enumeration lstnr=listeners.elements();
        while(lstnr.hasMoreElements())
            tree.addTreeSelectionListener((TreeSelectionListener)lstnr.nextElement());
		setCursor(oldCur);	
		SwingUtilities.invokeLater(new Runnable() 
								   {
									   public void run() 
										   {
											   _updateUI();
										   }
								   }
								   );
        //revalidate();
		System.out.println("Done building tree");
    }

	protected void _updateUI()
	{
		scrollpane.setViewportView(tree);
		cards.show(this, "Tree");
	}
	
}










