/************************************************************************************
* Copyright (C) 1998 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/

package utils;

public class Base64
{
  private static boolean debug=false;
  private static char mask[]={128, 64, 32, 16, 8, 4, 2, 1};
  private static char table[]={ 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/' };

  protected static char value(char val)
  {
    char res=0;

    if (val>='A'&&val<='Z')
      res=(char)(val-'A');
    else if (val>='a'&&val<='z')
      res=(char)(26+val-'a');
    else if (val>='0'&&val<='9')
      res=(char)(52+val-'0');
    else if (val=='+')
      res=(char)62;
    else if (val=='/')
      res=(char)63;
    else
      System.err.println("\nbad char value "+val+" ("+Integer.toBinaryString(val)+")\n");
    if (debug&&(res!=0))
      System.out.println("converted "+val+" to "+res);
    return res;
  }

  protected static StringBuffer cleanUp(String data)
  {
    StringBuffer res=new StringBuffer(data.length());
    int idx=0;
    int next=0;
    while(idx!=-1)
    {
      next=data.indexOf('\n', idx);
      if (next!=-1)
      {
	res.append(data.substring(idx, next));
	idx=next+1;
      }
      else
      {
	res.append(data.substring(idx));
	idx=-1;
      }
    }
    return res;
  } 

  public static char[] decode(StringBuffer in)
  {
    return decode(in.toString());
  }

  public static char[] decode(String data_in)
  {
    if (data_in==null||data_in.length()<4)
      return null;
    int size;
    char res[];
    StringBuffer data=cleanUp(data_in);
    int max=data.length();
    if (data.charAt(max-2)=='='&&data.charAt(max-1)=='=')
    {
      max-=2;
      size=(3*max)/4;
    }
    else if (data.charAt(max-1)=='=')
      size=(3*(--max))/4;
    else
      size=(3*max)/4;
    res=new char[size];
    char cur=0;
    char bit_mask=2;
    char shift_count=0;
    int index=0;
    int target=0;
    char cur_char=value(data.charAt(index++));

    while(index<=max)
    {
      cur<<=1;
      if ((cur_char&mask[bit_mask++])!=0)
	cur|=1;
      if (++shift_count==8)
      {
	res[target++]=cur;
	cur=0;
	shift_count=0;	
      }
      if (bit_mask==8)
      {
	bit_mask=2;
	if (index<max)
	  cur_char=value(data.charAt(index));
	index++;
      }
    }
    return res;
  }

  public static StringBuffer encode(char[] data)
  {
    
    if (data==null||data.length==0)
      return new StringBuffer("");
    StringBuffer result=new StringBuffer((int)(data.length*4/3+1));
    char cur=data[0];
    int end=data.length*8; //#bits to process
    char bit_offset=0;
    char mask_offset=0;
    char res=0;
    int index=0;
    int nb_char=0;
    int cur_char=0;
    StringBuffer bin=null;
    if (debug)
      bin=new StringBuffer();
    while(index<end)
    {
      res<<=1;
      if ((cur&mask[bit_offset++])!=0)
      {
	res|=1;
	if (debug)
	  bin.append("1");
      }
      else if (debug)
	bin.append("0");
      
      if (++mask_offset==6)
      {
	if (debug)
	{
	  System.out.println(bin.toString()+" => "+Integer.toString(res));
	  bin.setLength(0);
	}
	mask_offset=0;
	result.append(table[res]);
	res=0;
	if (++nb_char==76)
	{
	  result.append('\n');
	  nb_char=0;
	}
      }
      if (bit_offset==8)
      {
	bit_offset=0;
	if (++cur_char<data.length)
	  cur=data[cur_char];
      }
      index++;
    }
    if (mask_offset!=0)
    {
      while(mask_offset++<6)
      {
	res<<=1; //pad with 0 until with have 6 bits.
	if (debug)
	  bin.append("0");
      }
      if (debug)
	System.out.println(bin.toString()+" => "+Integer.toString(res));
      result.append(table[res]);
      if (++nb_char==76)
      {
	result.append('\n');
	nb_char=0;
      }
    }
    int rem=end%24;
    switch(rem)
    {
    case 8:
      result.append('=');
      if (++nb_char==76)
      {
	result.append('\n');
	nb_char=0;
      }
      //fallthrough
    case 16:
      result.append('=');
      break;
    default:
      break;
    }
    result.append('\n');
    return result;
  }
}



